/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean.workflow;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import edu.uiuc.ncsa.cet.bean.CETBean;
import edu.uiuc.ncsa.cet.bean.PersonBean;

public class WorkflowToolBean extends CETBean implements Serializable {
    /** Used for serialization of object */
    private static final long           serialVersionUID = 1L;

    /** Title of the workflow tool */
    private String                      title            = "";                                    //$NON-NLS-1$

    /** Description of the workflow tool */
    private String                      description      = "";                                    //$NON-NLS-1$

    /** Version of the workflow tool */
    private String                      version          = "1";                                   //$NON-NLS-1$

    /** Previous version of the workflow tool */
    private CETBean                     previousVersion  = null;

    /** Date the workflow tool is created */
    private Date                        date             = new Date();

    /** implementation, this is executor specific. */
    private String                      implementation   = "";                                    //$NON-NLS-1$

    /** creator of the workflow tool */
    private PersonBean                  creator          = null;

    /**
     * List of contributors to the workflow tool.
     * 
     */
    private Set<PersonBean>             contributors     = new HashSet<PersonBean>();

    /**
     * List of inputs to the workflow tool.
     * 
     */
    private List<WorkflowToolDataBean>  inputs           = new ArrayList<WorkflowToolDataBean>();

    /**
     * List of outputs to the workflow tool.
     * 
     */
    private List<WorkflowToolDataBean>  outputs          = new ArrayList<WorkflowToolDataBean>();

    /**
     * List of parameters to the workflow tool.
     * 
     */
    private List<WorkflowParameterBean> parameters       = new ArrayList<WorkflowParameterBean>();

    /**
     * Executor of the tool.
     * 
     */
    private String                      executor         = null;

    /**
     * Create a new instance of the workflow tool.
     */
    public WorkflowToolBean() {
    }

    /**
     * Return the title of the workflow tool.
     * 
     * @return title of the workflow tool
     */
    public String getTitle() {
        return title;
    }

    /**
     * Sets the title of the workflow tool
     * 
     * @param title
     *            sets the title of the workflow tool.
     * 
     */
    public void setTitle(String title) {
        this.title = title;
    }

    /**
     * Return the description of the workflow tool.
     * 
     * @return description of the workflow tool
     */
    public String getDescription() {
        return description;
    }

    /**
     * Sets the description of the workflow tool
     * 
     * @param description
     *            sets the description of the workflow tool
     * 
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Return the date when the workflow tool was created.
     * 
     * @return date the workflow tool was created.
     */
    public Date getDate() {
        return date;
    }

    /**
     * Sets the date when the workflow tool was created.
     * 
     * @param date
     *            sets the date when the workflow tool was created.
     * 
     */
    public void setDate(Date date) {
        this.date = date;
    }

    /**
     * Return the implementation of the tool
     * 
     * @return implementation of the tool.
     */
    public String getImplementation() {
        return implementation;
    }

    /**
     * Sets the implementation of the tool.
     * 
     * @param implementation
     *            sets the implementation of the tool.
     * 
     */
    public void setImplementation(String implementation) {
        this.implementation = implementation;
    }

    /**
     * Return the executor of the tool
     * 
     * @return executor of the tool.
     */
    public String getExecutor() {
        return executor;
    }

    /**
     * Sets the executor of the tool.
     * 
     * @param executor
     *            sets the executor of the tool.
     * 
     */
    public void setExecutor(String executor) {
        this.executor = executor;
    }

    /**
     * Return the version of the workflow tool.
     * 
     * @return version of the workflow tool.
     */
    public String getVersion() {
        return version;
    }

    /**
     * Sets the version of the workflow tool.
     * 
     * @param version
     *            sets the version of the workflow tool.
     * 
     */
    public void setVersion(String version) {
        this.version = version;
    }

    /**
     * Return the previous version of the workflow tool.
     * 
     * @return previous version of the workflow tool.
     */
    public CETBean getPreviousVersion() {
        return previousVersion;
    }

    /**
     * Sets the previous version of the workflow tool.
     * 
     * @param previousVersion
     *            sets the previous version of the workflow tool.
     * 
     */
    public void setPreviousVersion(CETBean previousVersion) {
        this.previousVersion = previousVersion;
    }

    /**
     * Return the PersonBean that is the creator of the workflow tool
     * 
     * @return PersonBean that represents the creator
     */
    public PersonBean getCreator() {
        return creator;
    }

    /**
     * Sets the PersonBean that represents the creator of the workflow tool.
     * 
     * @param creator
     *            sets the PersonBeans that represents the creator of the
     *            workflow tool.
     * 
     */
    public void setCreator(PersonBean creator) {
        this.creator = creator;
    }

    /**
     * Return the set of PersonBeans that represents those that are contributors
     * to the workflow tool.
     * 
     * @return set of PersonBeans that represents all the users that have
     *         contributes to the workflow tool.
     */
    public Set<PersonBean> getContributors() {
        return this.contributors;
    }

    /**
     * Set the set of PersonBeans that represents those that contributed to the
     * workflow tool.
     * 
     * @param contributors
     *            the set of contributors to the workflow tool.
     */
    public void setContributors(Collection<PersonBean> contributors) {
        this.contributors.clear();
        if (contributors != null) {
            this.contributors.addAll(contributors);
        }
    }

    /**
     * Add the contributor to the set of contributors to the workflow tool.
     * 
     * @param contributor
     *            the PersonBean of the contributor to be added.
     */
    public void addContributor(PersonBean contributor) {
        if (contributor != null) {
            this.contributors.add(contributor);
        }
    }

    /**
     * Remove the contributor from the set of contributors to the worflow tool.
     * 
     * @param contributor
     *            the PersonBean of the contributor to be removed.
     */
    public void removeContributor(PersonBean contributor) {
        this.contributors.remove(contributor);
    }

    /**
     * Return the collection of datasets that are inputs to the tool.
     * 
     * @return collection of DatasetListBean that represents all the inputs to
     *         the workflow tool.
     */
    public List<WorkflowToolDataBean> getInputs() {
        return this.inputs;
    }

    /**
     * Set the list of datasets that make up the inputs to the tool.
     * 
     * @param inputs
     *            the list of datasets to be used as inputs.
     */
    public void setInputs(List<WorkflowToolDataBean> inputs) {
        this.inputs.clear();
        if (inputs != null) {
            this.inputs.addAll(inputs);
        }
    }

    /**
     * Return the input associated with this id.
     * 
     * @param id
     *            the id whose input to return.
     * @return input associated with the id.
     */
    public WorkflowToolDataBean getInput(String id) {
        if ((id == null) || id.equals("")) { //$NON-NLS-1$
            return null;
        }

        for (WorkflowToolDataBean dlb : inputs) {
            if (id.equals(dlb.getId())) {
                return dlb;
            }
        }

        return null;
    }

    /**
     * Add the data to list of inputs to the workflow tool.
     * 
     * @param data
     *            the data to be added as input.
     */
    public void addInput(WorkflowToolDataBean data) {
        inputs.add(data);
    }

    /**
     * Remove the data from list of inputs to the workflow tool.
     * 
     * @param data
     *            the data to be removed as input.
     */
    public void removeInput(WorkflowToolDataBean data) {
        inputs.remove(data);
    }

    /**
     * Return the collection of datasets that are outputs to the tool.
     * 
     * @return collection of beans that represents all the outputs to the
     *         workflow tool.
     */
    public List<WorkflowToolDataBean> getOutputs() {
        return this.outputs;
    }

    /**
     * Set the list of datasets that make up the outputs to the tool.
     * 
     * @param outputs
     *            the list of datasets to be used as outputs.
     */
    public void setOutputs(List<WorkflowToolDataBean> outputs) {
        this.outputs.clear();
        if (outputs != null) {
            this.outputs.addAll(outputs);
        }
    }

    /**
     * Return the output associated with this id.
     * 
     * @param id
     *            the id whose output to return.
     * @return output associated with the id.
     */
    public WorkflowToolDataBean getOutput(String id) {
        if ((id == null) || id.equals("")) { //$NON-NLS-1$
            return null;
        }

        for (WorkflowToolDataBean dlb : outputs) {
            if (id.equals(dlb.getId())) {
                return dlb;
            }
        }

        return null;
    }

    /**
     * Add the data to list of outputs to the workflow tool.
     * 
     * @param data
     *            the DatasetBean to be added as output.
     */
    public void addOutput(WorkflowToolDataBean data) {
        outputs.add(data);
    }

    /**
     * Remove the data from list of outputs to the workflow tool.
     * 
     * @param data
     *            the DatasetBean to be removed as output.
     */
    public void removeOutput(WorkflowToolDataBean data) {
        outputs.remove(data);
    }

    /**
     * Return the collection of parameters of the tool.
     * 
     * @return collection of WorkflowParameterBean that represents all the
     *         parameters to the workflow tool.
     */
    public List<WorkflowParameterBean> getParameters() {
        return this.parameters;
    }

    /**
     * Return the parameter associated with this id.
     * 
     * @param id
     *            the id whose parameter to return.
     * @return parameter associated with the id.
     */
    public WorkflowParameterBean getParameter(String id) {
        if ((id == null) || id.equals("")) { //$NON-NLS-1$
            return null;
        }

        for (WorkflowParameterBean wpb : parameters) {
            if (id.equals(wpb.getId())) {
                return wpb;
            }
        }

        return null;
    }

    /**
     * Sets the collection of parameters of the tool.
     * 
     * @param parameters
     *            collection of WorkflowParameterBean that represents all the
     *            parameters to the workflow tool.
     */
    public void setParameters(List<WorkflowParameterBean> parameters) {
        this.parameters.clear();
        if (parameters != null) {
            this.parameters.addAll(parameters);
        }
    }

    /**
     * Add the parameter from the set of parameters to the worflow tool.
     * 
     * @param parameter
     *            the parameter to be added.
     */
    public void addParameter(WorkflowParameterBean parameter) {
        this.parameters.add(parameter);
    }

    /**
     * Remove the parameter from the set of parameters to the worflow tool.
     * 
     * @param parameter
     *            the parameter to be removed.
     */
    public void removeParameter(WorkflowParameterBean parameter) {
        this.parameters.remove(parameter);
    }
}
